/*
 * Created on October 21, 2005
 *
 */
package com.sunlabs.archsim.fleet.ships.selector;
import com.sunlabs.archsim.base.common.Parameters;
import com.sunlabs.archsim.gasp.Component;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;




/**
 * @author mh193489
 *
 * Selector ship for fleet.
 *
 * Selector consumes data from either In1 or In2, depending on the value on 
 * the Select input. The data on Select is an integer, 1 or 2, specifying the
 * corresponding branch.
 * 
 * Upon receiving a valid data (i.e. 1 or 2) on Select, 
 * the Selector outputs on Out the data consumed from the corresponding In 
 * terminal. 
 * 
 * Upon receiving a LAST on its Select input, it will wait for 
 * a LAST to arrive on In1 and In2, consuming and ignoring all non LAST data. 
 * 
 * Once both LASTs have arrived the Selector returns to its normal operation.
 */
public class Selector extends Component {
    
    InputTerminal in1;
    OutputTerminal ackIn1;
    InputTerminal in2;
    OutputTerminal ackIn2;
    InputTerminal inSelect;
    OutputTerminal ackInSelect;
    
    OutputTerminal out;
    InputTerminal ackOut;
    
    /** accumulator action hidden in the ship */
    SelectorAction adder;
    
    
    public Selector(String name, Parameters params) {
        super(name, null);
        
		// process the parameters
		if (params != null) {
			for (int i= 0; i<params.size(); i++) {
				String pName= params.getName(i);
				String sValue= params.getValue(i);
		
				director.error("Component: " + name + ", id= " + getID() 
						+ ": Attribute " + pName + " with value " + sValue
						+ " is ignored");					
		
			} // for
		} // if params != null       
        
		buildSelector();
		
    } // Selector
    
    public void buildSelector() {
        // create terminals for the Selector component
        in1= new InputTerminal(SelectorDefaults.IN1, this);
        ackIn1= new OutputTerminal(SelectorDefaults.ACK_IN1, this);
        
        in2= new InputTerminal(SelectorDefaults.IN2, this);
        ackIn2= new OutputTerminal(SelectorDefaults.ACK_IN2, this);
        
        inSelect= new InputTerminal(SelectorDefaults.IN_SELECT, this);
        ackInSelect= new OutputTerminal(SelectorDefaults.ACK_IN_SELECT, this);
        
        out= new OutputTerminal(SelectorDefaults.OUT, this);
        ackOut= new InputTerminal(SelectorDefaults.ACK_OUT, this);
        
        // create the action performing the adding
        adder= new SelectorAction(SelectorDefaults.ADDER, this);
        
        // attach terminals to the action
        adder.attach(in1);
        adder.attach(ackIn1);
        adder.attach(in2);
        adder.attach(ackIn2);
        adder.attach(inSelect);
        adder.attach(ackInSelect);
        adder.attach(out);
        adder.attach(ackOut);
           	
    }
    
    /**
     * Get a terminal by name.
     * Return null if no such terminal exists.
     */
    public Terminal getTerminal(String name) {
        if (name.equals(in1.getBaseName())
        || name.equals(in1.getBaseAlias()))
            return in1;
        
        else if (name.equals(ackIn1.getBaseName())
        || name.equals(ackIn1.getBaseAlias()))
            return ackIn1;

                if (name.equals(in2.getBaseName())
        || name.equals(in2.getBaseAlias()))
            return in2;
        
        else if (name.equals(ackIn2.getBaseName())
        || name.equals(ackIn2.getBaseAlias()))
            return ackIn2;

                if (name.equals(inSelect.getBaseName())
        || name.equals(inSelect.getBaseAlias()))
            return inSelect;
        
        else if (name.equals(ackInSelect.getBaseName())
        || name.equals(ackInSelect.getBaseAlias()))
            return ackInSelect;

        else if (name.equals(out.getBaseName())
        || name.equals(out.getBaseAlias()))
            return out;
        
        else if (name.equals(ackOut.getBaseName())
        || name.equals(ackOut.getBaseAlias()))
            return ackOut;
        
        else
            return null;
        
    } // getTerminal
    
} // Selector;


