/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */

/*
 * Created on Apr 6, 2005
 *
 */
package com.sunlabs.archsim.tests.merge;

import com.sunlabs.archsim.fleet.components.switchFabric.Merge;
import com.sunlabs.archsim.gasp.State;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;

/**
 * @author ib27688
 *
 */
public class MergeTestFactory {
	private MergeTestFactory() {}
	
	public static MergeTest please(String name) {
		
		int numInputs= MergeTestGlobal.NUM_INPUTS;
		
		MergeTest test= new MergeTest(name);
		
		// create the array for the sources
		MergeTestSource[] src= new MergeTestSource[numInputs];
//		 prepare the arrays for the states
		State[] dataInState= new State[numInputs];
		State[] ackDataInState= new State[numInputs];
		
		OutputTerminal[] srcDataTerm= new OutputTerminal[numInputs];
		InputTerminal[] srcAckDataTerm= new InputTerminal[numInputs];

		Merge merge= new Merge(MergeTestGlobal.MERGE, numInputs);		
		
		// now make the sources and the states
		for (int i=0; i< numInputs; i++) {
			src[i]= new MergeTestSource(
					MergeTestGlobal.SOURCE+MergeTestGlobal.DELIMITER+i, 
					i, (i+1)*1000);
			test.addMember(src[i]);
			
			// make terminals for the source
			srcDataTerm[i]= new OutputTerminal(
					MergeTestGlobal.SRC_OUT+MergeTestGlobal.DELIMITER+i);
			test.addMember(srcDataTerm[i]);
			
			srcAckDataTerm[i]= new InputTerminal(
					MergeTestGlobal.SRC_ACK_OUT+MergeTestGlobal.DELIMITER+i);
			test.addMember(srcAckDataTerm[i]);
			
			// attach the terminals to the source
			src[i].attach(srcDataTerm[i]);
			src[i].attach(srcAckDataTerm[i]);
			
			dataInState[i]= new State(
					MergeTestGlobal.DATA_IN_STATE
					+ MergeTestGlobal.DELIMITER + i);
			test.addMember(dataInState[i]);
			
			// attach terminals to the data in state
			dataInState[i].attach(srcDataTerm[i]);
			dataInState[i].attach(merge.getIn(i));
			
			
			ackDataInState[i]= new State(
					MergeTestGlobal.ACK_DATA_IN_STATE
					+ MergeTestGlobal.DELIMITER + i);
			test.addMember(ackDataInState[i]);
			
			// attach terminals to the ack state
			ackDataInState[i].attach(srcAckDataTerm[i]);
			ackDataInState[i].attach(merge.getAckIn(i));
		}
		
		
		// make the states form the merge to the sink
		State dataOutState= new State(MergeTestGlobal.DATA_OUT_STATE);
		test.addMember(dataOutState);
		State ackDataOutState= new State(MergeTestGlobal.ACK_DATA_OUT_STATE);
		test.addMember(ackDataOutState);
		
		// mnake the sink
		MergeTestSink sink= new MergeTestSink(MergeTestGlobal.SINK);
		test.addMember(sink);
			
		// make sink terminals
		InputTerminal sinkInTerm= new InputTerminal(MergeTestGlobal.SINK_IN);
		test.addMember(sinkInTerm);
		
		OutputTerminal sinkAckInTerm= new OutputTerminal(MergeTestGlobal.SINK_ACK_IN);
		test.addMember(sinkAckInTerm);

		//attach sink terminals
		sink.attach(sinkInTerm);
		sink.attach(sinkAckInTerm);
		
		// attach terminals to the states
		dataOutState.attach(merge.getOut());
		dataOutState.attach(sinkInTerm);
		
		ackDataOutState.attach(merge.getAckOut());
		ackDataOutState.attach(sinkAckInTerm);
		
		// initialize input states
		for (int i=0; i<numInputs; i++) {
			// ackDataInState[i].setActive(null);
			ackDataInState[i].setInitiallyActive(null);
		}
		// ackDataOutState.setActive(null);
		ackDataOutState.setInitiallyActive(null);
		
		return test;
	} // please
	
}
